package usherbrooke.ift630;

import java.lang.Long;
import java.util.ArrayList;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.Executors;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.LinkedBlockingQueue;

public class App {
	static private int numBusPerTrip		= 1;
	static private int numTrips				= 1;
	static private int numBusses			= numTrips * numBusPerTrip;
	static private int numStopPerBus		= 8;
	static private int numStops				= numBusses * numStopPerBus;
	static private int numPassengersPerBus	= 10;
	static private int numPassengersPerStop = 5;
	//static private int numPassengers		= numPassengersPerStop * numStops;
	static private int numThreadsBus		= 5;
	static private int numThreadsStop		= 2;
	static private int timeBetweenStops 	= 5;
	static private int timeEmbark			= 2;

	public static void main(String[] args) {
		ExecutorService threadsBus = Executors.newFixedThreadPool(numThreadsBus);
		ExecutorService threadsStop = Executors.newFixedThreadPool(numThreadsStop);
		ExecutorService threadsPassenger = Executors.newFixedThreadPool(1); // doesnt need more! future balance
																			// themselfes out.
		ArrayList<Stop> stops = new ArrayList<Stop>();
		ArrayList<Bus> busses = new ArrayList<Bus>();
		ArrayList<Trip> trips = new ArrayList<Trip>();
		ArrayList<Passenger> passengers = new ArrayList<Passenger>();
		BlockingQueue<BusInformationMessage> blockingQueue = new LinkedBlockingQueue<BusInformationMessage>();

		if (numStopPerBus > numStops) {
			System.out.println("More stops per bus than stops.");
			return;
		}

		// make stop" + name + " reached stop " + nextStop.getStopName() + "!"s
		for (int k = 0; k < numStops; k++) {
			Stop s = new Stop(k, numPassengersPerStop);
			s.setBlockingQueue(blockingQueue);

			stops.add(s);
			s.setThreadPool(threadsStop);
			threadsStop.submit(s);
		}

		// make trpis
		for (int k = 0 ; k < numTrips ; k++) {
			Stop s;
			ArrayList<Stop> s_list = new ArrayList<Stop>();
			while (s_list.size() < numStopPerBus) {
				do {
					s = stops.get((int) (Math.random() * numStops));
				} while (s_list.contains(s));
				s_list.add(s);
			}

			Trip t = new Trip(k, s_list);
			trips.add(t);
		}

		// make busses
		for (int k = 0; k < numBusses; k++) {
			Bus b = new Bus(trips.get(k % trips.size()), k, timeBetweenStops, timeEmbark, numPassengersPerBus);
			b.setBlockingQueue(blockingQueue);

			busses.add(b);
		}

		// make passenger
		int idPassenger = 0;
		for (Bus b : busses) {
			for (Stop start : b.getStops()) {

				// make sure passenger can leave.
				if (start == b.getTerminus())
					continue;

				int idx = b.getStops().indexOf(start);
				if (idx == -1 || start == b.getTerminus())
					continue;

				for (int k = 0; k < (int) (Math.random() * numPassengersPerStop); k++) {
					Stop dest = null;

					dest = b.getStops().get(idx + 1 + (int) Math.round(Math.random() * (numStopPerBus - 2 - idx)));

					Passenger p = new Passenger(idPassenger, start, dest);
					passengers.add(p);
					start.addPassenger(p);
					idPassenger++;

					threadsPassenger.submit(p);
				}
			}
		}

		// pretty print!
		for (Bus b : busses) {
			b.printDetails();
		}

		System.out.println("-----".repeat(5) + " START " + "-----".repeat(5));

		// start bus thread
		// (here so init logs are clean)
		for (Bus b : busses) {
			threadsBus.submit(b);
		}

		// block bus pool
		threadsBus.shutdown();
		try {
			threadsBus.awaitTermination(Long.MAX_VALUE, TimeUnit.DAYS);
		} catch (InterruptedException e) {}

		// block stop and passenger pool once bus have finished
		threadsStop.shutdown();
		threadsPassenger.shutdown();
		try {
			threadsStop.awaitTermination(Long.MAX_VALUE, TimeUnit.DAYS);
		} catch (InterruptedException e) {}

		try {
			threadsPassenger.awaitTermination(Long.MAX_VALUE, TimeUnit.DAYS);
		} catch (InterruptedException e) {}
	}
}
