package usherbrooke.ift630;

import java.util.concurrent.Future;
import java.util.concurrent.Executors;
import java.util.concurrent.ExecutorService;

public class Passenger extends Thread {

	private String name;
	private int id;
	private Stop dest;
	private Stop start;
	private ExecutorService ex;

	Passenger(int id, Stop start, Stop dest) {
		this.id = id;
		this.name = "Passenger " + id;
		this.dest = dest;
		this.start = start;
		this.ex = Executors.newFixedThreadPool(1); // one for each future.
	}

	public void run() {
		try {
			embark();
			disembark();
		} catch (Exception e) {
			Logger.getInstance().print(id, "[PASSENGER] " + e.toString());
		}
	}

	// embark people once thread notified
	private synchronized Future<Void> embark() {
		return ex.submit(() -> {
			synchronized (this) {
				wait();
			}
			Logger.getInstance().print(id, "\t[PASSENGER] " + name + " embarking at " + start.getStopName());
			return null;
		});
	}

	// disembark people once thread notified another time.
	private synchronized Future<Void> disembark() {
		return ex.submit(() -> {
			synchronized (this) {
				wait();
			}
			Logger.getInstance().print(id, "\t[PASSENGER] " + name + " leaving at " + dest.getStopName());
			return null;
		});
	}

	public Stop getStart() {
		return start;
	}

	public Stop getDest() {
		return dest;
	}

	public String getPassengerName() {
		return name;
	}

	// Pretty print, with color!
	public void printDetails(int color, int indent) {
		Logger.getInstance().print(color,
				"\t".repeat(indent) + "---".repeat(3) + " Passenger details " + "---".repeat(3));
		Logger.getInstance().print(color,
				"\t".repeat(indent) + name + " start: " + start.getStopName() + " dest: " + dest.getStopName());
	}

	// Pretty print, with indent!
	public void printDetails(int indent) {
		Logger.getInstance().print(id, "\t".repeat(indent) + "---".repeat(3) + " Passenger details " + "---".repeat(3));
		Logger.getInstance().print(id,
				"\t".repeat(indent) + name + " start: " + start.getStopName() + " dest: " + dest.getStopName());
	}

	// Pretty print!
	public void printDetails() {
		printDetails(0);
	}
}
