package usherbrooke.ift630;

import java.util.concurrent.BlockingQueue;
import java.util.concurrent.ExecutorService;
import java.util.ArrayList;

public class Stop extends Thread {
	private String name;
	private int id;
	private int maxCapacity;
	private int indent = 0;
	private ArrayList<Passenger> passengers;
	private ExecutorService threads;
	private BlockingQueue<BusInformationMessage> blockingQueue;

	// print bus info times.
	private void displayInfos(int indent) throws InterruptedException, ExitException {
		BusInformationMessage info = null;

		// lock queue, get head
		synchronized (blockingQueue) {
			info = blockingQueue.poll();
		}
		// get infos
		try {
			try {
				int time = info.getTimeToStop(this) / 1000;
				// if everything is alright, print
				Logger.getInstance().print(id,
						"\t".repeat(indent) + "[STOP] " + info.getBus(this).getNameBus() + " arrives in " + time + "s");
				// else, catch
			} catch (NullPointerException e) {
			}

		} catch (UnauthorizedException e) {
			// if unauth, requeue message.
			synchronized (blockingQueue) {
				blockingQueue.add(info);
			}
		}

		// loop, and ensure that every stop is reactive
		synchronized (this) {
			// i dont have access to virtual threads, and futures are for the next question.
			// to not hog allof the pool's capacity,
			// i have to manualy change threads in action. This is done by submiting this
			// thread, waiting a bit (dont blow up our CPU) then exit. This ensure that
			// another thread takes the slot, if it there is one.
			sleep(10);
			threads.submit(this);
			throw new ExitException();
		}
	}

	@Override
	public void run() {
		try {
			for (;;) {
				displayInfos(indent);
			}
		} catch (InterruptedException | ExitException e) {
		}
	}

	Stop(int id, int maxCapacity) {
		this.id = id;
		this.name = "Stop " + id;
		this.passengers = new ArrayList<Passenger>();
		this.maxCapacity = maxCapacity;
	}

	public void addPassenger(Passenger p) {
		passengers.add(p);
	}

	// return all passenger that stops at a stop in the list
	public ArrayList<Passenger> getPassengerByDest(Trip t, int indexStart) {
		ArrayList<Passenger> res = new ArrayList<Passenger>();
		// for all given stop
		for (Stop s : t.getStops(indexStart)) {
			// for all passenger at this stop
			for (Passenger p : passengers) {
				// if they stop at this stop, add to res
				if (p.getDest() == s) {
					res.add(p);
				}
			}
		}

		return res;
	}

	public String getStopName() {
		return name;
	}

	public int getCurrentCapacity() {
		return passengers.size();
	}

	public int getMaxCapacity() {
		return maxCapacity;
	}

	public void printDetails(int color, int indent) {
		Logger.getInstance().print(color, "\t".repeat(indent) + "---".repeat(3) + " Stop details " + "---".repeat(3));
		Logger.getInstance().print(color, "\t".repeat(indent) + name);

		for (Passenger p : passengers) {
			p.printDetails(color, indent + 1);
		}
	}

	public void printDetails(int indent) {
		Logger.getInstance().print(id, "\t".repeat(indent) + "---".repeat(3) + " Stop details " + "---".repeat(3));
		Logger.getInstance().print(id, "\t".repeat(indent) + name);

		for (Passenger p : passengers) {
			p.printDetails(indent + 1);
		}
	}

	public void printDetails() {
		printDetails(0);
	}

	public synchronized void removePassenger(Passenger p) {
		if(passengers.remove(p))
			synchronized (p) {
				p.notify();
			}
	}

	public void setIndent(int indent) {
		this.indent = indent;
	}

	public void setBlockingQueue(BlockingQueue<BusInformationMessage> q) {
		blockingQueue = q;
	}

	public void setThreadPool(ExecutorService threads) {
		this.threads = threads;
	}
}
